import { getIronSession } from "iron-session";
import { cookies } from "next/headers";
import getSessionOptions from "@/app/api/auth/session/options";
import SessionPayload from "@/app/api/auth/session/payload";
import AppDataSource from "@/app/_data/source/app-data-source";
import { Ticket } from "@/app/_data/entities/Ticket";


export async function GET(request: Request, { params }: { params: { ticket_id: number }}) {
  try {
    const { ticket_id } = params;
    const session = await getIronSession<SessionPayload>(cookies(), getSessionOptions());
    if (!session.TwoStepAuthentication) {
      return Response.json(null);
    }
    if (!session.accountNo) {
      return Response.json({ message: `Please sign in before raise a ticket` })
    }

    const connection_crm = await AppDataSource.getConnection();
    const ticketRepo = connection_crm.getRepository(Ticket);
    // Get list of ticket raised by the user
    const ticket_list = await ticketRepo.createQueryBuilder('ticket')
      .select(['id','ticket_id', 'category_of_request', 'nature_of_request', 'additional_info','message','created_date'])
      .where("ticket.ticket_id = :ticket_id" , { ticket_id: ticket_id })
      .andWhere("ticket.customer_account_no = :customer_account_no" ,{customer_account_no:session.accountNo })
      .getRawOne();
    if(!ticket_list){return Response.json("No Ticket is Present.");}
  return Response.json(ticket_list)
  }
  catch (error) {
    return Response.json(null, { status: 500 });
  }
}

