import { getIronSession } from "iron-session";
import { cookies } from "next/headers";
import getSessionOptions from "../auth/session/options";
import SessionPayload from "../auth/session/payload";
import AppDataSource from "@/app/_data/source/app-data-source";
import { Ticket } from "@/app/_data/entities/Ticket";
import { User } from "@/app/_data/entities/User";
import { assignUser } from "./assign_memeber";


export async function POST(request: Request) {
  try {
    const session = await getIronSession<SessionPayload>(
      cookies(),
      getSessionOptions()
    );
    if (!session.accountNo) {
      return Response.json({ message: `Please sign in before raise a ticket` })
    }
    if (!session.TwoStepAuthentication) {
      return Response.json(null);
    }
    const mantisBT_Token =process.env.MantisBT_Token || "";  //MantisBT Token
    
    const body = await request.json();
    //request_category, nature_of_request, issue, message 
    const { summary, description, additional_information, steps_to_reproduce } = body;
    const project = { id: 1 };
    const requestBody = { ...body, project };
    const response_for_issue_ticket = await (await fetch(`http://localhost/crm_internal/api/rest/issues/`, {
      method: "POST",
      headers:
        { "Content-Type": "application/json", "Authorization": mantisBT_Token },
      body: JSON.stringify(requestBody)
    }))
    if (response_for_issue_ticket.status === 403) {
      return Response.json({ message: 'Invalid Token' }, { status: 403 })
    }
    if (response_for_issue_ticket.status === 400) {
      return Response.json({ message: 'Project not specified' }, { status: 400 })
    }
    if (response_for_issue_ticket.status === 404) {
      return Response.json({ message: 'Project not found' }, { status: 404 })
    }
    if (response_for_issue_ticket.status === 429) {
      return Response.json({ message: 'Please raised ticket after sometime.' }, { status: 429 })
    }

    // Assign backend team to the ticket
    const issue_id = parseInt(response_for_issue_ticket.statusText);
    const assign_person = await assignUser(summary);
    const response_for_assignee_issue_ticket = await (await fetch(`http://localhost/CRM_Internal/api/rest/issues/${issue_id}`, {
      method: "PATCH",
      headers:
        { "Content-Type": "application/json", "Authorization": mantisBT_Token },
      body: JSON.stringify({handler :{ id : assign_person }})
    }))
    if (response_for_assignee_issue_ticket.status === 403) {
      return Response.json({ message: 'Invalid Token' }, { status: 403 })
    }
    if(response_for_assignee_issue_ticket.status ==200)
    {
      console.log("Assignee is assign against the ticket");
    }
    ///CRM database management 
    const connection_crm = await AppDataSource.getConnection();
    const userRepo = connection_crm.getRepository(User);
    const user_table = await userRepo.createQueryBuilder('user')
      .select(['first_name', 'middle_name', 'last_name'])
      .where("user.bank_account_no = :AccountNo", { AccountNo: session.accountNo })
      .getRawOne();

    const user_name = (user_table.first_name ? user_table.first_name + ' ' : '') +
      (user_table.middle_name ? user_table.middle_name + ' ' : '') +
      (user_table.last_name ? user_table.last_name + ' ' : '');
      
    const ticketRepo = connection_crm.getRepository(Ticket);
    const ticket = new Ticket();
    ticket.ticket_id = parseInt(response_for_issue_ticket.statusText);
    ticket.category_of_request = summary;
    ticket.nature_of_request = description;
    ticket.additional_info = additional_information
    ticket.message = steps_to_reproduce;
    ticket.created_by = user_name;
    ticket.customer_account_no = Number(session.accountNo);
    ticket.created_date=new  Date().toLocaleString("en-IN");
    
    await ticketRepo.save(ticket);
    return Response.json({ message: `Ticket No:${response_for_issue_ticket.statusText} created successfully` })
  }
  catch (error) {
    return Response.json(null, { status: 500 });
  }
}


export async function GET(request: Request) {
  try {
    const session = await getIronSession<SessionPayload>(cookies(), getSessionOptions());
    if (!session.TwoStepAuthentication) {
      return Response.json(null);
    }
    if (!session.accountNo) {
      return Response.json({ message: `Please sign in before raise a ticket` })
    }
    const connection_crm = await AppDataSource.getConnection();
    const connection_mantis = await AppDataSource.getMantisConnection();
    const userRepo = connection_crm.getRepository(User);
    // Get full name of user
    const user_table = await userRepo.createQueryBuilder('user')
      .select(['first_name', 'middle_name', 'last_name'])
      .where("user.bank_account_no = :AccountNo", { AccountNo: session.accountNo })
      .getRawOne();
    const user_name = (user_table.first_name ? user_table.first_name + ' ' : '') +
      (user_table.middle_name ? user_table.middle_name + ' ' : '') +
      (user_table.last_name ? user_table.last_name + ' ' : '');
    
    const ticketRepo = connection_crm.getRepository(Ticket);
    // Get list of ticket raised by the user
    const ticket_list = await ticketRepo.createQueryBuilder('ticket')
      .select(['id','ticket_id', 'category_of_request', 'nature_of_request', 'created_date','additional_info'])
      .where("ticket.created_by = :created_by" , { created_by: user_name })
      .andWhere("ticket.customer_account_no = :customer_account_no" ,{customer_account_no:session.accountNo })
      .getRawMany();

    if(!ticket_list){return Response.json('No Details Found')}
    const ticket_ids = ticket_list.map(ticket => ticket.ticket_id);  // collect the ticket_id (ticket_id store the mantisBT's bug id)

    // Mantis-database fetching for "status" & "message"
    const queryRunner = connection_mantis.createQueryRunner();
    await queryRunner.connect();
    let ticketResolution: { [key: number]: string } = {};
    let ticketMessage: { [key: number]: string[] } = {};

    for (const id of ticket_ids) {
      // For status of ticket
      const ticket_resolution = await queryRunner.query('SELECT resolution,handler_id FROM "kccb_bug" WHERE id = $1', [id]);
      const ticket_status =ticket_resolution[0]?.resolution
      const ticket_handler =ticket_resolution[0]?.handler_id
      let status :string ='';
      if(ticket_status === 10 && ticket_handler === 0){status = 'Open'}
      if(ticket_handler > 0){status = 'In Progress'}
      if(ticket_status === 20){status='Resolved'}
      if(ticket_status === 30){status='Reopen'}
      ticketResolution[id] = status   
      // For Message of ticket
      const ticket_note = await queryRunner.query('SELECT bugnote_text_id FROM "kccb_bugnote" WHERE bug_id = $1', [id]);
      const ticket_note_ids =ticket_note.map((item:{bugnote_text_id:number}) =>item.bugnote_text_id) // collect all ids from kccb_bugnote table for a issue

      for(const note_id of ticket_note_ids ){
        const ticket_note_text = await queryRunner.query('SELECT note FROM "kccb_bugnote_text" WHERE id = $1', [note_id]);
           
        if (!ticketMessage[id]) {
          ticketMessage[id] = [];
        }
        ticketMessage[id].push(...ticket_note_text);
      }
    }
  // merge message & status in the response
  const response_for_issue_ticket = ticket_list.map(item =>
  (
    {
      ...item,status:ticketResolution[item.ticket_id]||null,message: ticketMessage[item.ticket_id]||null
    }
  )
  )
  queryRunner.release();
  return Response.json(response_for_issue_ticket)
  }
  catch (error) {
    return Response.json(null, { status: 500 });
  }
}

